"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var path = require("path");
var Progress = require("progress");
function progressBar(installer, stream) {
    if (stream === void 0) { stream = process.stderr; }
    var progress;
    var tick = function (phase, last, numberOfTicks) {
        if (last === void 0) { last = null; }
        if (numberOfTicks === void 0) { numberOfTicks = 1; }
        return progress.tick(numberOfTicks, { phase: phase, last: last ? "(" + last + ")" : '' });
    };
    installer.on('install_targets_identified', function (installTargets) {
        var packSteps = _.uniqBy(_.flatMap(installTargets, function (target) { return target.sources; }), function (_) { return _.directory; }).length;
        var phaseSteps = 2;
        var installSteps = _.sum(_.flatMap(installTargets, function (target) { return target.sources.length; }));
        var steps = installSteps + packSteps + phaseSteps;
        progress = new Progress("install-local: [:bar] :phase:last", { total: steps, stream: stream });
    });
    installer.on('packing_start', function (_) { return tick('packing'); });
    installer.on('packed', function (dir) { return tick('packing', path.basename(dir)); });
    installer.on('install_start', function (_) { return tick('installing'); });
    installer.on('installed', function (target) { return tick('installing', path.basename(target.directory), target.sources.length); });
    installer.on('install_end', function () { return progress.terminate(); });
}
exports.progressBar = progressBar;
//# sourceMappingURL=progressBar.js.map